;; --------------------------------
;; Presentación de entorno PlanTron
;; --------------------------------
;;  Este fichero recoge una serie de instrucciones
;;  que guían el aprendizaje del entorno de agentes
;;  PlanTron (ver fichero README).
;;
;;  Puedes ir copiando sobre una consola las
;;  instrucciones aquí descritas.
;;


;; Para cargar el entorno
;; ----------------------

;; Arrancamos en consola el CMUCL lisp con 'cmucl'
#|

cmucl

|#
;; También es posible usar SBCL lisp


;; Cargamos el entorno en el lugar en que esté,
;; en el CC de la ETSII se encuentra en el
;; directorio /scratch/plantron
;;

(load "/scratch/plantron/pt-loader")

;; ¡Atención! Cargar siempre desde scratch, ya
;; que pueden haber modificaciones en los códigos.
;; Comprobar versión contenido fichero VERSION o
;; ejecutar en ahora (version-pt)



;; Partida con humano
;; ------------------

;; Movimientos de teclas (teclado numérico):
;;   8: arriba, 2:abajo, 6: derecha, 4:izquierda

;; ¡Atención! Asegurarse antes de jugar que el
;; tamaño de la terminal es al menos de 80x24
;; (VT100)

(pt:partida)


; La expresión anterior es equivalente a:

(pt:partida "AGENTE-HUMANO")


; Descripción de funciones y variables con describe

(describe 'pt:partida)


; Información del agente

(pt::info-jug "AGENTE-HUMANO")



;; Partida con agente mitad
;; ------------------------

(pt:partida "AGENTE-MITAD")

;; Más rápido, opción :sleep-cs <milisegundos/turno>

(pt:partida "AGENTE-MITAD" :sleep-cs 1)


;; Generar plantas en modo menos virulento,
;; variable pt-agentes-genplc:*var-sigm-hacia*

(setf pt-agentes-genplc:*var-sigm-hacia* 4)

(pt:partida "AGENTE-MITAD" :sleep-cs 1)


;; Generar plantas en modo mas virulento

(setf pt-agentes-genplc:*var-sigm-hacia* 128)

(describe 'pt-agentes-genplc:*var-sigm-hacia*)

(pt:partida "AGENTE-MITAD" :sleep-cs 3)



;;
;; Consultar jugadores disponibles
;; -------------------------------

(pt:jug)

(describe 'pt:jug)

(pt:jug :full t)



;; Partida con agente lados
;; ------------------------

;; Generar plantas en modo menos virulento

(setf pt-agentes-genplc:*var-sigm-hacia* 4)

(pt:partida "AGENTE-LADOS" :sleep-cs 1)

;; Generar plantas en modo mas virulento

(setf pt-agentes-genplc:*var-sigm-hacia* 128)

(pt:partida "AGENTE-LADOS" :sleep-cs 5)



;; Partida con agente espiral
;; --------------------------

;; Generar plantas en modo menos virulento

(setf pt-agentes-genplc:*var-sigm-hacia* 4)

(pt:partida "AGENTE-ESPIRAL" :sleep-cs 1)

;; Generar plantas en modo mas virulento

(setf pt-agentes-genplc:*var-sigm-hacia* 128)

(pt:partida "AGENTE-ESPIRAL" :sleep-cs 5)



;; Partida con agente muralla
;; --------------------------

(pt:partida "AGENTE-MURALLA" :sleep-cs 1)



;; Partida con agente sin-miedo
;; ----------------------------

;; Generar plantas en modo menos virulento

(setf pt-agentes-genplc:*var-sigm-hacia* 4)

(pt:partida "AGENTE-SIN-MIEDO" :sleep-cs 5)

;; Generar plantas en modo mas virulento

(setf pt-agentes-genplc:*var-sigm-hacia* 128)

(pt:partida "AGENTE-SIN-MIEDO" :sleep-cs 1)



;; Partida con agente cobarde
;; --------------------------

;; Generar plantas en modo menos virulento

(setf pt-agentes-genplc:*var-sigm-hacia* 4)

(pt:partida "AGENTE-COBARDE" :sleep-cs 5)

;; Generar plantas en modo mas virulento

(setf pt-agentes-genplc:*var-sigm-hacia* 128)

(pt:partida "AGENTE-COBARDE" :sleep-cs 5)



;; Partida con agente hormiga borracha
;; -----------------------------------

(pt:partida "AGENTE-HORMIGA-BORRACHA" :sleep-cs 10)



;; Partidas con dos agentes
;; ------------------------

(pt:partida "AGENTE-MITAD" "AGENTE-ESPIRAL" :sleep-cs 1)


(pt:partida "AGENTE-MITAD" "AGENTE-MURALLA" :sleep-cs 1)


(pt:partida "AGENTE-MURALLA" "AGENTE-SIN-MIEDO" :sleep-cs 1)


(pt:partida "AGENTE-SIN-MIEDO" "AGENTE-COBARDE" :sleep-cs 1)



;; Partida con trazas de cosultas, opción :vis <nivel>
;; ---------------------------------------------------

(pt:partida "AGENTE-COBARDE" :sleep-cs 5 :vis 2)

(pt:partida "AGENTE-COBARDE" :sleep-cs 1 :vis 2)



;; Partida con otros generadores de plantas, opción :gen-pla
;; --------------------------------------------------------

; Generador de plantas aleatorio

(pt:partida "AGENTE-COBARDE" :sleep-cs 1 :gen-pla 2 :vis 2)


; Generador de plantas en cúmulos no incremental,
; detener con Ctrl-C y comando TOP

(pt:partida "AGENTE-MURALLA" :sleep-cs 1 :gen-pla 1 :vis 2)



;; Cargar y ejecutar un nuevo agente
;; ---------------------------------

(pt:load-ag "agentes-aluXXXX.lisp")

(describe 'pt:load-ag)

(pt:jug)

(pt:partida "ALUXXXX-MITAD" :sleep-cs 1)



;; Pintar el tablero en otra terminal, opción :dev-vis
;; ---------------------------------------------------

; Localizar antes con comando 'w' el dispositivo
; asociado a otra terminal existente,
; por ejemplo "/dev/pts/3"

(pt:partida "AGENTE-ESPIRAL" :sleep-cs 1 :dev-vis "/dev/pts/3")



;; Partidas paso a paso
;; --------------------

(setf pt-vis:*step* t)

(pt:partida "AGENTE-ESPIRAL" :vis 2 :dev-vis "/dev/pts/3")

(describe 'pt-vis:*step*)



;; Partidas de todos los agentes no interactivos
;; ---------------------------------------------

(pt:partida-todos :sleep-cs 5)

(describe 'pt:partida-todos)



; Resultados de 5 partidas del agente espiral
; ejecutadas sin visualizador (ojo, puede tardar)

(pt:partida-todos '("AGENTE-ESPIRAL") :num 5 :vis nil)



;; Trabajo con repositorio arena
;; -----------------------------

; Mostrar los agentes en repositorio arena

(pt-arena:ls)

(describe 'pt-arena:ls)

(pt-arena:ls "*" t)


; Chequear y enviar agentes al repositorio arena

(pt-arena:send "agentes-aluXXXX.lisp")


; Cargar agente del repositorio arena

(pt-arena:load-ag "agentes-aluXXXX")


; Tablas de resultados

(pt-arena:hof)

(describe 'pt-arena:hof)



;; Salir del cmucl lisp
;; --------------------

(quit)
