;; Programa: Package con agentes pasivos con distintas
;;           utilidades para asistir a agentes jugadores de
;;           PlanTron
;;
;; Asignatura: Introducción a la Inteligencia Artificial
;;             ETSII - ULL
;; Práctica: (sistemas de producción)
;; Profesor: Patricio García (patricio AT etsii.ull.es)
;; Fecha:    18/05/2007
;; Comentarios:
;;
;;           Los agentes definidos utilizan el entorno de juego
;;           PlanTron, definido principalmente por los packages
;;           PT, PT-BASE Y ENVIO.
;;
;;           Las funciones accesibles más importantes son:
;;
;;           ....
;;
;;           Ver la documentación existente para cada una de ellas.
;;
;;           Para su utilización es aconsejable usar el cargador
;;           del entorno de juego PlanTron: "pt-loader.lisp".
;;
;;
;; Dependencias:
;;
;;           Hace uso del paquete que contiene la interface (ENVIO)
;;           con el control del juego (PT-BASE).
;;
;;           Hace uso de los paquetes para crear sistemas de
;;           prudocción (SHELL) y búsquedas genéricas
;;           (GENERIC-SEARCH)
;;
;; Modificaciones:
;;
;;           - Ver 0.1 (18/05/2007): Primera versión.
;;
;;           - Ver 0.1.1 (25/05/2007): Añadido keys :servicio-id
;;             Reparado error en PLANTA-CERCA que a veces devolvía
;;             menos plantas de las pedidas e inicializaba mal
;;
;;
;; Por hacer:
;;           - Corrección de Bugs
;;             ------------------
;;           - Funciones #:GENERIC-SEARCH, #:REGLA, #:LIMPIA-REGLAS
;;             e #:HIJOS no cogen bien el nombre del pacakge en la
;;             definición de AGENTE
;;           - PLANTA-CERCA a veces devuelve menos plantas de las
;;             pedidas
;;           - Eliminar wernings de compilación
;;
;;           - Mejoras en funcionalidades
;;             --------------------------
;;           - Incorporar gestión de energía de muertes y creaciones
;;             de plantas
;;           - Generalizar planta cerca para que pueda encontrar no
;;             solo plantas, sino incluso otros objetos
;;           - Que se pueda indicar de alguna manera los objetos a
;;             seguir (lista de id's)
;;           - Identificador de contador y/ contador por id agente
;;           - Mejorar interfaces con shell y generic-search para
;;             permitir que varios agentes las usen en la misma
;;             partida
;;           - Cambiar todos los :servicio-id por key concreta y
;;             ver si implementar en llamadas
;;           - Generar agente que reciba e imprima :mensaje en un
;;             dispositivos configurable ("/dev/pts/?")
;;
;;           - Mejoras en estilo de programación
;;             ---------------------------------
;;
;;


;;
;; Definición del package
;; ----------------------

(defpackage "PT-UTIL"
  (:nicknames "PT-UT")
  (:use "COMMON-LISP")
  (:export "AGENTES" "VERSION" "PLANTA-CERCA" "PLANTAS-CERCA-INI" "PLANTA-CERCA-NUEVA-PLANTA" "PLANTA-CERCA-MUERE-PLANTA" "DISTANCIA-CITY" "CONSULTA-CONTADOR" "INICIA-CONTADOR" "CONTADOR-INI" "*SISTEMA*" "LIMPIA-REGLAS" "REGLA" "HIJOS" "*SEARCH-QUEUE*" "GENERIC-SEARCH" "*EVAL-FN*" "SPLICE" "PRIORITY-MERGE-FN" "PRIORITY-DEQUEUE-FN"))


(in-package "PT-UTIL")



;;
;; Constante versión
;; -----------------

(defconstant version "0.1.1"
  "Versión del package PT-UTIL.")


;;
;; Constante con lista de agentes presentes en este package
;; --------------------------------------------------------

(defconstant agentes
  `((:servicio-id (planta-cerca distancia-city)
     :name "PLANTA-CERCA"
     :inicio plantas-cerca-ini
     :seg-npla planta-cerca-nueva-planta
     :seg-mpla planta-cerca-muere-planta
     :tipo-ag envio:pasivo)
    (:name "CONTADOR"
     :documentation "Utilidad para cronometrar tiempos."
     :servicio-id (consulta-contador inicia-contador)
     :inicio contador-ini
     :tipo-ag envio:pasivo)
    (:name "SHELL"
     :documentation ,(documentation (find-package "SHELL") T)
     :servicio-id (regla limpia-reglas hijos)
     :tipo-ag envio:pasivo)
    (:name "GENERIC-SEARCH"
     :documentation ,(documentation (find-package "GENERIC-SEARCH") T)
     :servicio-id generic-search
     :tipo-ag envio:pasivo)
    (:name "A_STAR"
     :documentation ,(documentation (find-package "A_STAR") T)
     :servicio-id (priority-a-merge-fn priority-a-dequeue-fn)
     :tipo-ag envio:pasivo))
  "Agentes con utilidades para asistira a agentes jugadores de PlanTron.")


;;
;; Variables para agente buscador de plantas cercanas
;; --------------------------------------------------

(defvar lis-plantas nil
  "Lista de plantas actuales")


;;
;; Funciones de apoyo para agente buscador de plantas cercanas
;; -----------------------------------------------------------

(defun distancia-city (pos1 pos2)
  "Calcula la distancia city block entre POS1 y POS2"
  (+ (abs (- (first pos1) (first pos2)))
     (abs (- (second pos1) (second pos2)))))


(defun inserta-ord (pos dist id lis-ord num)
  "Inserta ordenada por distancia la planta en la lista LIS-ORD."
  (do* ((ant nil lis)
	(lis lis-ord (rest lis))
	(cont num (1- cont)))
      
      ((or (zerop cont)
	   (endp lis)
	   (<= dist (second (first lis))))
       (if (not (zerop cont))
	   (if ant
	       (setf (rest ant) (cons (list pos dist id)
				      (rest ant)))
	     (setf lis-ord (cons (list pos dist id)
				 lis-ord))))
       lis-ord)))


;;
;; Agente buscador de plantas cercanas
;; -----------------------------------

(defun plantas-cerca-ini (jug tipo-juego)
  "Inicializa el agente buscador de plantas cercanas."
  (declare (ignore jug tipo-juego))
  (setf lis-plantas nil)
  t)


(defun planta-cerca-nueva-planta (id pos)
  "Seguimiento de creación de nueva planta del agente buscador de plantas cercanas."
  (push (list pos id) lis-plantas))


(defun planta-cerca-muere-planta (id1 id2 pos)
  "Seguimiento de muerte de planta del agente buscador de plantas cercanas."
  (declare (ignore id2))
  (setf lis-plantas (delete-if #'(lambda (elm)
				   (and (equal (first elm) pos)
					(eq (second elm) id1)))
			       lis-plantas
			       :count 1)))

(defun planta-cerca (pos &optional (num 1) id)
  "Agente buscador de plantas cercanas. Dada la posición POS busca las NUM plantas más cercanas a esta posición y las devuelve en una lista donde cada planta viene representada por una lista con su posición (lista con coordenada x y coordenada y), la distancia city-block a POS y su energía. Si ID es distinto de NIL busca solo entre plantas generadas por el generador de plantas con identificador ID."
  (let (lis-ord)
    (dolist (planta lis-plantas (subseq lis-ord 0 (min num (length lis-ord))))
      (if (or (not id) (eq id (second planta)))
	  (setf lis-ord (inserta-ord (first planta)
				     (distancia-city pos (first planta))
				     (second planta)
				     lis-ord
				     num))))))


;;
;; Variables para agente buscador de plantas cercanas
;; --------------------------------------------------

(defvar counter nil
  "Vairable contador.")

(defvar limite-cs-turno nil
  "Límite en centésimas de segundo para decidir movimiento en un turno para partida actual.")


;;
;; Funciones de apoyo para agente contador
;; ---------------------------------------

(defun ini-counter (limite-cs)
  "Inicializa el contador a LIMITE-CS centésimas de segundo."
  (setf counter
	(if (and limite-cs (> limite-cs 0))
	    (+ (get-internal-real-time)
	       (/ (* limite-cs internal-time-units-per-second)
		  100)))))

(defun test-counter (&optional hang)
  "Chequea si el contador llegado al límite, esperando si HANG es T."
  (if counter
      (do ()
	  ((> (get-internal-real-time) counter) t)
	(if (not hang)
	    (return nil)))
    hang))


;; Agente contador de tiempo
;; -------------------------

(defun contador-ini (jug tipo-juego)
  "Inicializa el agente contador de tiempo."
  (declare (ignore jug))
  (setf limite-cs-turno (getf tipo-juego :sleep-cs)))

(defun inicia-contador (&optional (limite-cs limite-cs-turno))
  "Inicializa contador del agente contador de tiempo a LIMITE-CS centésimas de segundo"
  (ini-counter limite-cs))

(defun consulta-contador (&optional hang)
  "Consulta al agente contador de tiempo si se ha cumplido el tiempo previamente inicializado con INICIA-CONTADOR, esperando si HANG es T."
  (test-counter hang))


;; Agente de apoyo de sistemas de producción
;; -----------------------------------------

(defconstant lis-exp-shell '(shell:*sistema* shell:limpia-reglas shell:regla shell:hijos))
(shadowing-import lis-exp-shell)
(export lis-exp-shell)


(defconstant lis-exp-generic-search '(generic-search:*search-queue* generic-search:generic-search generic-search:*eval-fn* generic-search:splice priority-merge-fn generic-search:priority-dequeue-fn))
(shadowing-import lis-exp-generic-search)
(export lis-exp-generic-search)


(defconstant lis-exp-a_star '(a_star:*coste-acum-fn* a_star:*igual-est-fn* a_star:criba-iguales a_star:fusion a_star:priority-a-merge-fn a_star:priority-a-dequeue-fn))
(shadowing-import lis-exp-a_star)
(export lis-exp-a_star)

